@@ -23,15 +23,16 @@ import (
 	"sync"
 	"time"
 
-	"git.zabbix.com/ap/mongodb/plugin/handlers"
-	"git.zabbix.com/ap/plugin-support/log"
-	"git.zabbix.com/ap/plugin-support/tlsconfig"
-	"git.zabbix.com/ap/plugin-support/uri"
-	"git.zabbix.com/ap/plugin-support/zbxerr"
 	"go.mongodb.org/mongo-driver/bson"
 	"go.mongodb.org/mongo-driver/mongo"
 	"go.mongodb.org/mongo-driver/mongo/options"
 	"go.mongodb.org/mongo-driver/mongo/readpref"
+	"golang.zabbix.com/plugin/mongodb/plugin/handlers"
+	"golang.zabbix.com/sdk/errs"
+	"golang.zabbix.com/sdk/log"
+	"golang.zabbix.com/sdk/tlsconfig"
+	"golang.zabbix.com/sdk/uri"
+	"golang.zabbix.com/sdk/zbxerr"
 )
 
 const (
@@ -70,13 +71,12 @@ type MongoQuery struct {
 
 // ConnManager is thread-safe structure for manage connections.
 type ConnManager struct {
-	sync.Mutex
-	connMutex   sync.Mutex
-	connections map[connKey]*MongoConn
-	keepAlive   time.Duration
-	timeout     time.Duration
-	Destroy     context.CancelFunc
-	log         log.Logger
+	connectionsMu sync.Mutex
+	connections   map[connKey]*MongoConn
+	keepAlive     time.Duration
+	timeout       time.Duration
+	Destroy       context.CancelFunc
+	log           log.Logger
 }
 
 type connKey struct {
@@ -95,17 +95,27 @@ func (conn *MongoConn) DB(name string) handlers.Database {
 	return &MongoDatabase{Database: conn.session.Client().Database(name)}
 }
 
-func (conn *MongoConn) DatabaseNames() (names []string, err error) {
+// DatabaseNames returns a list of database names.
+func (conn *MongoConn) DatabaseNames() ([]string, error) {
 	conn.checkConnection()
 
-	return conn.session.Client().ListDatabaseNames(context.Background(), bson.D{})
+	names, err := conn.session.Client().
+		ListDatabaseNames(context.Background(), bson.D{})
+	if err != nil {
+		return nil, errs.Wrap(err, "failed to list database names")
+	}
+
+	return names, nil
 }
 
 func (conn *MongoConn) Ping() error {
 	Impl.Debugf("executing ping for address: %s", conn.addr)
 	result := conn.session.Client().Database("admin").RunCommand(
 		context.Background(),
-		&bson.D{{Key: "ping", Value: 1}, {Key: "maxTimeMS", Value: conn.GetMaxTimeMS()}},
+		&bson.D{
+			{Key: "ping", Value: 1},
+			{Key: "maxTimeMS", Value: conn.GetMaxTimeMS()},
+		},
 	)
 
 	err := result.Err()
@@ -141,18 +151,29 @@ func (d *MongoDatabase) Run(cmd, result interface{}) error {
 // Collection is an interface to access to the collection struct.
 
 // Find shadows *mgo.Collection to returns a Query interface instead of *mgo.Query.
-func (c *MongoCollection) Find(query interface{}, opts ...*options.FindOptions) (handlers.Query, error) {
+func (c *MongoCollection) Find( //nolint:ireturn
+	query any,
+	opts ...*options.FindOptions,
+) (handlers.Query, error) {
 	cursor, err := c.Collection.Find(context.Background(), query, opts...)
 	if err != nil {
-		return nil, err
+		return nil, errs.Wrap(err, "failed to execute find query")
 	}
 
 	return &MongoQuery{Cursor: cursor}, nil
 }
 
 // FindOne shadows *mgo.Collection to returns a Query interface instead of *mgo.Query.
-func (c *MongoCollection) FindOne(query interface{}, opts ...*options.FindOneOptions) handlers.Query {
-	return &MongoQuery{SingleResult: c.Collection.FindOne(context.Background(), query, opts...)}
+func (c *MongoCollection) FindOne( //nolint:ireturn
+	query any,
+	opts ...*options.FindOneOptions,
+) handlers.Query {
+	return &MongoQuery{
+		SingleResult: c.Collection.FindOne(
+			context.Background(),
+			query,
+			opts...),
+	}
 }
 
 func (q *MongoQuery) Count() (int, error) {
@@ -182,7 +203,10 @@ func (q *MongoQuery) GetSingle(result interface{}) error {
 }
 
 // NewConnManager initializes connManager structure and runs Go Routine that watches for unused connections.
-func NewConnManager(keepAlive, timeout, hkInterval time.Duration, logger log.Logger) *ConnManager {
+func NewConnManager(
+	keepAlive, timeout, hkInterval time.Duration,
+	logger log.Logger,
+) *ConnManager {
 	ctx, cancel := context.WithCancel(context.Background())
 
 	connMgr := &ConnManager{
@@ -199,27 +223,69 @@ func NewConnManager(keepAlive, timeout, hkInterval time.Duration, logger log.Log
 }
 
 // GetConnection returns an existing connection or creates a new one.
-func (c *ConnManager) GetConnection(uri uri.URI, params map[string]string) (*MongoConn, error) {
-	c.Lock()
-	defer c.Unlock()
-
-	ck := createConnKey(uri, params)
-
-	conn := c.get(ck)
+func (c *ConnManager) GetConnection(
+	connURI uri.URI, //nolint:gocritic
+	params map[string]string,
+) (*MongoConn, error) {
+	ck := createConnKey(connURI, params)
 
+	conn := c.getConn(ck)
 	if conn != nil {
-		c.log.Tracef("connection found for host: %s", uri.Host())
+		c.log.Tracef("connection found for host: %s", connURI.Host())
 
 		return conn, nil
 	}
 
-	return c.create(ck, params)
+	conn, err := c.create(ck, params)
+	if err != nil {
+		return nil, errs.Wrap(err, "failed to create new connection")
+	}
+
+	return c.setConn(ck, conn), nil
+}
+
+// getConn returns a connection with given uri if it exists and also updates
+// lastTimeAccess, otherwise returns nil.
+func (c *ConnManager) getConn(ck connKey) *MongoConn { //nolint:gocritic
+	c.connectionsMu.Lock()
+	defer c.connectionsMu.Unlock()
+
+	conn, ok := c.connections[ck]
+	if !ok {
+		return nil
+	}
+
+	conn.updateAccessTime()
+
+	return conn
+}
+
+func (c *ConnManager) setConn(
+	ck connKey, //nolint:gocritic
+	conn *MongoConn,
+) *MongoConn {
+	c.connectionsMu.Lock()
+	defer c.connectionsMu.Unlock()
+
+	existingConn, ok := c.connections[ck]
+	if ok {
+		conn.session.EndSession(context.Background())
+		conn.session.Client().Disconnect(context.Background()) //nolint:errcheck,gosec
+
+		Impl.Debugf("Closed unused connection: %s", ck.uri.Addr())
+
+		return existingConn
+	}
+
+	c.connections[ck] = conn
+
+	return conn
 }
 
 // closeUnused closes each connection that has not been accessed at least within the keepalive interval.
 func (c *ConnManager) closeUnused() {
-	c.connMutex.Lock()
-	defer c.connMutex.Unlock()
+	c.connectionsMu.Lock()
+	defer c.connectionsMu.Unlock()
 
 	for ck, conn := range c.connections {
 		if time.Since(conn.lastTimeAccess) > c.keepAlive {
@@ -233,13 +299,13 @@ func (c *ConnManager) closeUnused() {
 
 // closeAll closes all existed connections.
 func (c *ConnManager) closeAll() {
-	c.connMutex.Lock()
+	c.connectionsMu.Lock()
 	for uri, conn := range c.connections {
 		conn.session.EndSession(context.Background())
 		_ = conn.session.Client().Disconnect(context.Background())
 		delete(c.connections, uri)
 	}
-	c.connMutex.Unlock()
+	c.connectionsMu.Unlock()
 	Impl.Debugf("Closed all connections")
 }
 
@@ -261,7 +327,10 @@ func (c *ConnManager) housekeeper(ctx context.Context, interval time.Duration) {
 }
 
 // create creates a new connection with given credentials.
-func (c *ConnManager) create(ck connKey, params map[string]string) (*MongoConn, error) {
+func (c *ConnManager) create(
+	ck connKey, //nolint:gocritic
+	params map[string]string,
+) (*MongoConn, error) {
 	opt, err := c.createOptions(ck.uri, params)
 	if err != nil {
 		return nil, err
@@ -287,33 +356,32 @@ func (c *ConnManager) create(ck connKey, params map[string]string) (*MongoConn,
 		return nil, err
 	}
 
-	c.connMutex.Lock()
-	defer c.connMutex.Unlock()
+	Impl.Debugf("Created new connection: %s", ck.uri.Addr())
 
-	c.connections[ck] = &MongoConn{
+	return &MongoConn{
 		addr:           ck.uri.Addr(),
 		timeout:        c.timeout,
 		lastTimeAccess: time.Now(),
 		session:        session,
-	}
-
-	Impl.Debugf("Created new connection: %s", ck.uri.Addr())
-
-	return c.connections[ck], nil
+	}, nil
 }
 
 // get returns a connection with given uri if it exists and also updates lastTimeAccess, otherwise returns nil.
-func (c *ConnManager) createOptions(uri uri.URI, params map[string]string) (*options.ClientOptions, error) {
+func (c *ConnManager) createOptions(
+	connURI uri.URI, //nolint:gocritic
+	params map[string]string,
+) (*options.ClientOptions, error) {
 	details, err := createTLS(params)
 	if err != nil {
 		return nil, err
 	}
 
 	opt := options.Client()
-	if uri.User() != "" {
+
+	if connURI.User() != "" {
 		creds := options.Credential{}
-		creds.Username = uri.User()
-		creds.Password = uri.Password()
+		creds.Username = connURI.User()
+		creds.Password = connURI.Password()
 		creds.PasswordSet = true
 		opt = opt.SetAuth(creds)
 	}
@@ -325,7 +393,7 @@ func (c *ConnManager) createOptions(uri uri.URI, params map[string]string) (*opt
 		}
 	}
 
-	opt.SetHosts([]string{uri.Addr()})
+	opt.SetHosts([]string{connURI.Addr()})
 	opt.SetDirect(true)
 	opt.SetConnectTimeout(c.timeout)
 	opt.SetServerSelectionTimeout(c.timeout)
@@ -372,10 +440,12 @@ func createTLS(params map[string]string) (*tlsconfig.Details, error) {
 	}
 
 	return &details, nil
-
 }
 
-func (c *ConnManager) setTLSConfig(opt *options.ClientOptions, details *tlsconfig.Details) error {
+func (c *ConnManager) setTLSConfig(
+	opt *options.ClientOptions,
+	details *tlsconfig.Details,
+) error {
 	var cfg *tls.Config
 	var err error
 
@@ -388,14 +458,17 @@ func (c *ConnManager) setTLSConfig(opt *options.ClientOptions, details *tlsconfi
 	case "verify_ca":
 		cfg, err = details.GetTLSConfig(true)
 		if err != nil {
-			return zbxerr.New("failed to get TLS config for verify_ca connection").Wrap(err)
+			return errs.Wrap(err, "failed to get TLS config for verify_ca connection")
 		}
 
-		cfg.VerifyPeerCertificate = tlsconfig.VerifyPeerCertificateFunc("", cfg.RootCAs)
+		cfg.VerifyPeerCertificate = tlsconfig.VerifyPeerCertificateFunc(
+			"",
+			cfg.RootCAs,
+		)
 	case "verify_full":
 		cfg, err = details.GetTLSConfig(false)
 		if err != nil {
-			return zbxerr.New("failed to get TLS config for verify_full connection").Wrap(err)
+			return errs.Wrap(err, "failed to get TLS config for verify_full connection")
 		}
 	}
 
@@ -404,9 +477,14 @@ func (c *ConnManager) setTLSConfig(opt *options.ClientOptions, details *tlsconfi
 	return nil
 }
 
-func (c *ConnManager) getRequiredTLSConfig(details *tlsconfig.Details) (*tls.Config, error) {
+func (c *ConnManager) getRequiredTLSConfig(
+	details *tlsconfig.Details,
+) (*tls.Config, error) {
 	if details.TlsCaFile != "" {
-		c.log.Warningf("server CA will not be verified for %s", details.TlsConnect)
+		c.log.Warningf(
+			"server CA will not be verified for %s",
+			details.TlsConnect,
+		)
 	}
 
 	clientCerts, err := details.LoadCertificates()
@@ -417,20 +495,6 @@ func (c *ConnManager) getRequiredTLSConfig(details *tlsconfig.Details) (*tls.Con
 	return &tls.Config{Certificates: clientCerts, InsecureSkipVerify: true}, nil
 }
 
-// get returns a connection with given uri if it exists and also updates lastTimeAccess, otherwise returns nil.
-func (c *ConnManager) get(ck connKey) *MongoConn {
-	c.connMutex.Lock()
-	defer c.connMutex.Unlock()
-
-	if conn, ok := c.connections[ck]; ok {
-		conn.updateAccessTime()
-
-		return conn
-	}
-
-	return nil
-}
-
 // updateAccessTime updates the last time a connection was accessed.
 func (conn *MongoConn) updateAccessTime() {
 	conn.lastTimeAccess = time.Now()
@@ -439,9 +503,8 @@ func (conn *MongoConn) updateAccessTime() {
 // checkConnection implements db reconnection.
 func (conn *MongoConn) checkConnection() {
 	if err := conn.Ping(); err != nil {
-		//TODO: implement reconnect
-		// conn.session.Refresh()
-		Impl.Debugf("Attempt to reconnect: %s", conn.addr)
+		Impl.Errf("Failed to ping connection - %q", conn.addr)
+		Impl.Debugf("Reconnect logic unimplemented")
 	}
 }
 
